@@ -1,5 +1,5 @@
 /*************************************************************
- * Unit:    raster            release 0.31                   *
+ * Unit:    raster            release 0.32                   *
  * Purpose: General manipulation n dimensional matrices      *
  *          n = 1, 2 and 3.			             *
  * Modul:   raster.cc                                        *
@@ -99,7 +99,7 @@ void Raster1DAbstract::Get(Raster1DAbstract &R1) const
 int i, maxi;
 int shift;
 
- maxi = Size1D>R1.Size1D?Size1D:R1.Size1D;
+ maxi = (Size1D>R1.Size1D) ? Size1D : R1.Size1D;
 
  if(R1.GetPlanes()==GetPlanes())
    {
