@@ -132,12 +132,12 @@ return result;
 
 string CheckPlainString(const string & command)
 {
-int i,state,num=0;
+int state,num=0;
 char ch;
 string plain;
 
  state=0;
- for(i=0; i<length(command); i++)
+ for(unsigned i=0; i<length(command); i++)
    {
    ch = command[i];
    switch(state)
@@ -276,7 +276,7 @@ FILE *F;
 void WriteCodeGenerator(doublelist & WP2LaTeX, const char *Table, const char *Switch)
 {
 FILE *F, *Fcp;
-int i, j;
+unsigned i, j;
 doublelist SimpleTable;
 unsigned TablePosStart;
 
@@ -523,7 +523,7 @@ FILE *F;
 codepage internal;
 codepage cp852, cp1250, cp1251, cp1252, cp1253, cp1254, cp1255, cp1276, cpkam,
          cpwp4a, cpwp5, cpwp5_cz, cpwp6,
-	 iso_8859_1, iso_8859_2, iso_8859_3, iso_8859_4, iso_8859_5,
+	 iso_8859_1, iso_8859_2, iso_8859_3, iso_8859_4, iso_8859_5, iso_8859_6,
          unicode,koi8cs,koi8r,
 	 mtef2,
          MAC_ROMAN;
@@ -627,6 +627,8 @@ int TimeoutCounter;
  if(iso_8859_4.name) {free(iso_8859_4.name);iso_8859_4.name=strdup("iso_8859_4");}
  LoadCodePage("is8859_5",iso_8859_5,0);
  if(iso_8859_5.name) {free(iso_8859_5.name);iso_8859_5.name=strdup("iso_8859_5");}
+ LoadCodePage("is8859_6",iso_8859_6,0);
+ if(iso_8859_6.name) {free(iso_8859_6.name);iso_8859_6.name=strdup("iso_8859_6");}
 
 
  //if(heapcheck()<=0)
@@ -700,6 +702,7 @@ WriteTranslator(cp1250,cpkam,trn,F);
  BuildTranslator(iso_8859_3,internal,F);	iso_8859_3.Erase();
  BuildTranslator(iso_8859_4,internal,F);	iso_8859_4.Erase();
  BuildTranslator(iso_8859_5,internal,F);	iso_8859_5.Erase();
+ BuildTranslator(iso_8859_6,internal,F);	iso_8859_6.Erase();
 
  BuildTranslator(cp852,internal,F);		cp852.Erase();
  BuildTranslator(cp1250,internal,F);		cp1250.Erase();
@@ -714,6 +717,7 @@ WriteTranslator(cp1250,cpkam,trn,F);
  BuildTranslator(unicode,internal,F);		// no erase, will be needed
 
 	// *************** Output codepage generator *****************
+ for(int i=128;i<=159;i++) unicode.InsertCharDesc(i,"Control Code",false);
  BuildTranslator(internal,unicode, F);		unicode.Erase();
  BuildTranslator(internal,cp1276, F);		cp1276.Erase();
  BuildTranslator(internal,symbol, F);		// no erase, will be needed
