@@ -1,5 +1,5 @@
 /***************************************************************************
- * Unit:    raster            release 0.26                                 *
+ * Unit:    raster            release 0.32                                 *
  * Purpose: Prototypes for general manipulation n dimensional matrices     *
  * Modul:   raster.cc                                                      *
  * Licency: GPL or LGPL                                                    *
@@ -15,7 +15,7 @@
 #include "stringa.h"
 
 
-#define RAS_IMG_VERSION (0x200 | 67)
+#define RAS_IMG_VERSION (0x200 | 75)
 
 /*Definition of errors while Loading/Saving File(s)*/
 #define ErrOK		  0
@@ -23,16 +23,19 @@
 #define ErrReadFile	 -2
 #define ErrWriteFile	 -3
 #define ErrBadFileFormat -4
+#define ErrMissingRaster -5
 
 #define ErrNoMem	-10
 
+extern const char ExifPreffix[6];
+
 
 typedef struct
-   {
+{
    uint8_t Red;
    uint8_t Green;
    uint8_t Blue;
-   } RGB_Record;
+} RGB_Record;
 
 
 class VectorImage;
@@ -48,7 +51,7 @@ public:
 
   mutable short UsageCount;
   void *Data;
-  unsigned DataSize;
+  uint32_t DataSize;
   string Name;
 };
 
@@ -82,7 +85,7 @@ public: Raster2DAbstract *Raster;
 
 	Image(void): VecImage(NULL)  {RotAngle=x=y=dx=dy=0;Raster=NULL;Next=NULL;Palette=NULL;};
 	Image(const Image & I, bool AllFrames=true);
-	Image(Raster2DAbstract *NewRaster): VecImage(NULL) {RotAngle=x=y=dx=dy=0;Next=NULL;Palette=NULL;if((Raster=NewRaster)!=NULL) Raster->UsageCount++;};
+	Image(Raster2DAbstract *NewRaster);
 	~Image(void) {Erase();};
 	IMAGE_TYPE ImageType(void) const;	// 0-none, 1-gray, 2-palette, 3-true color
 	bool isEmpty(void) const;
@@ -101,6 +104,7 @@ public: Raster2DAbstract *Raster;
 	void AttachPalette(APalette *NewPalette);
 	void AttachVecImg(VectorImage *NewVecImg);
 	void AttachProperty(PropertyItem *NewProp);
+        void AttachPropList(PropertyList &PropL);
 	};
 
 
