@@ -11,6 +11,21 @@ int SavePicture##SUFFIX(const char *Name,const Image &Img);\
 TImageFileHandler HANDLER_##SUFFIX(EXTENSION,
 
 
+#ifdef SupportAAI
+ ImplementReader(AAI,".AAI")
+ #if SupportAAI>=4 || SupportAAI==2
+   LoadPictureAAI,
+ #else
+   NULL,
+ #endif
+ #if SupportAAI>=3
+   SavePictureAAI);
+ #else
+   NULL);
+ #endif
+#endif
+
+
 #ifdef SupportART
  ImplementReader(ART,".ART")
  #if SupportART>=4 || SupportART==2
@@ -26,6 +41,21 @@ TImageFileHandler HANDLER_##SUFFIX(EXTENSION,
 #endif
 
 
+#ifdef SupportAVS
+ ImplementReader(AVS,".AVS")
+ #if SupportAVS>=4 || SupportAVS==2
+   LoadPictureAVS,
+ #else
+   NULL,
+ #endif
+ #if SupportAVS>=3
+   SavePictureAVS);
+ #else
+   NULL);
+ #endif
+#endif
+
+
 #ifdef SupportBMP
  ImplementReader(BMP,".BMP")
  #if SupportBMP>=4 || SupportBMP==2
@@ -76,6 +106,21 @@ TImageFileHandler HANDLER_##SUFFIX(EXTENSION,
 #endif
 
 
+#ifdef SupportCUR
+ ImplementReader(CUR,".CUR")
+ #if SupportCUR>=4 || SupportCUR==2
+	LoadPictureCUR,
+ #else
+	NULL,
+ #endif
+// #if SupportCUR>=3
+//	SavePictureCUR);
+// #else
+	NULL);
+// #endif
+#endif
+
+
 #ifdef SupportDIB
  ImplementReader(DIB,".DIB")
  #if SupportDIB>=4 || SupportDIB==2
@@ -551,10 +596,25 @@ TImageFileHandler TIFF_H(".TIFF",
    NULL,
  #endif
  #if SupportFTG>=3
-   SavePictureFTG);
+   SavePictureFTG,
  #else
-   NULL);
+   NULL,
  #endif
+   NULL, FILE_FORMAT_CAP_EXIF);
+#endif
+
+#ifdef SupportSGI
+  ImplementReader(SGI,".SGI")
+#if SupportSGI>=4 || SupportSGI==2
+	LoadPictureSGI,
+#else
+	NULL,
+#endif
+#if SupportSGI>=3
+	SavePictureSGI);
+#else
+	NULL);
+#endif
 #endif
 
 
@@ -571,3 +631,34 @@ TImageFileHandler TIFF_H(".TIFF",
 	NULL);
 #endif
 #endif
+
+
+#ifdef SupportWBMP
+  ImplementReader(WBMP,".WBMP")
+#if SupportWBMP>=4 || SupportWBMP==2
+	LoadPictureWBMP,
+#else
+	NULL,
+#endif
+#if SupportWBMP>=3
+	SavePictureWBMP);
+#else
+	NULL);
+#endif
+#endif
+
+
+#if defined(SupportWEBP) && SupportWEBP>=2
+ ImplementReader(WEBP,".WEBP")
+ #if SupportWEBP>=4 || SupportWEBP==2
+   LoadPictureWEBP,
+ #else
+   NULL,
+ #endif
+ #if SupportWEBP>=3
+   SavePictureWEBP,
+ #else
+   NULL,
+ #endif
+   NULL, FILE_FORMAT_CAP_EXIF);
+#endif
