@@ -1,5 +1,5 @@
 /******************************************************************************
- * program:     rasimg library 0.26                                           *
+ * program:     rasimg library 0.28                                           *
  * function:    Miscellaneous utilities for manipulation with images.         *
  * modul:       imgut.cc                                                      *
  * licency:     GPL or LGPL                                                   *
@@ -9,6 +9,7 @@
 #include<string.h>
 #include<stdlib.h>
 
+#include "../jobs/csext.h"
 #include "typedfs.h"
 #include "common.h"
 
@@ -160,20 +161,29 @@ void Image::Create(unsigned SizeX, unsigned SizeY, int Planes)
 if(Planes & ImageTrueColor)
   {
   if( (Raster=CreateRaster2DRGB(SizeX, SizeY, Planes&0xFF)) != NULL)
-	Raster->UsageCount++;
+	InterlockedIncrement(&Raster->UsageCount);
   }
 else
   {
   if( (Raster=CreateRaster2D(SizeX,SizeY,Planes&0xFF))!=NULL)
-	Raster->UsageCount++;
+	InterlockedIncrement(&Raster->UsageCount);
   }
 }
 
 
+Image::Image(Raster2DAbstract *NewRaster): VecImage(NULL)
+{
+ RotAngle=x=y=dx=dy=0;
+ Next=NULL;
+ Palette = NULL;
+ if((Raster=NewRaster)!=NULL) InterlockedIncrement(&Raster->UsageCount);
+}
+
+
 Image::Image(const Image & I, bool AllFrames)
 {
- if((Raster=I.Raster)!=NULL) Raster->UsageCount++;
- if((Palette=I.Palette)!=NULL) Palette->UsageCount++;
+ if((Raster=I.Raster)!=NULL) InterlockedIncrement(&Raster->UsageCount);
+ if((Palette=I.Palette)!=NULL) InterlockedIncrement(&Palette->UsageCount);
  if((VecImage=I.VecImage)!=NULL) VecImage->UsageCount++;
 
  if(!I.Properties.isEmpty())
@@ -203,10 +213,10 @@ Image::Image(const Image & I, bool AllFrames)
 
 Raster2DAbstract *Image::operator=(Raster2DAbstract *NewRaster)
 {
- x=y=dx=dy=0; 
+ x = y = dx = dy = 0;
  if(Raster==NewRaster) return(Raster);		//Identical assignment
  if(Raster!=NULL) Erase();
- if((Raster=NewRaster)!=NULL ) Raster->UsageCount++;
+ if((Raster=NewRaster)!=NULL ) InterlockedIncrement(&Raster->UsageCount);
  return(Raster);
 }
 
@@ -227,9 +237,9 @@ const Image *CurrImg = &I;
 	PImg = PImg->Next;
 
 CopyImg:if((PImg->Raster=CurrImg->Raster)!=NULL)
-		CurrImg->Raster->UsageCount++;
+	    InterlockedIncrement(&CurrImg->Raster->UsageCount);
 	if((PImg->Palette=CurrImg->Palette)!=NULL)
-		CurrImg->Palette->UsageCount++;
+	    InterlockedIncrement(&CurrImg->Palette->UsageCount);
 	PImg->x=CurrImg->x;PImg->y=CurrImg->y;
 	PImg->dx=CurrImg->dx;PImg->dy=CurrImg->dy;
 	PImg->RotAngle=CurrImg->RotAngle;
@@ -255,12 +265,12 @@ Image *I,*OldI;
    {
    if(I->Raster!=NULL)
       {
-      if(I->Raster->UsageCount--<=1) delete I->Raster;
+      if(InterlockedDecrement(&I->Raster->UsageCount)<=0) delete I->Raster;
       I->Raster = NULL;
       }
    if(I->Palette!=NULL)
       {
-      if(I->Palette->UsageCount--<=1) delete I->Palette;
+      if(InterlockedDecrement(&I->Palette->UsageCount)<=0) delete I->Palette;
       I->Palette = NULL;
       }
    if(I->VecImage!=NULL)
@@ -333,14 +343,14 @@ void Image::AttachRaster(Raster2DAbstract *NewRaster)
 {
   if(Raster==NewRaster) return;		// Raster is already attached
   if(Raster!=NULL)			// Detach previously attached raster
-   {
-   if(Raster->UsageCount--<=1) delete Raster;
-   Raster=NULL;
-   }
+  {
+    if(InterlockedDecrement(&Raster->UsageCount)<=0) delete Raster;
+    Raster=NULL;
+  }
   if(NewRaster!=NULL)			// Attach raster now
-   {
-   Raster=NewRaster; Raster->UsageCount++;
-   }
+  {
+    Raster=NewRaster; InterlockedIncrement(&Raster->UsageCount);
+  }
 }
 
 
@@ -348,14 +358,14 @@ void Image::AttachPalette(APalette *NewPalette)
 {
   if(Palette==NewPalette) return;		// Raster is already attached
   if(Palette!=NULL)			// Detach previously attached raster
-   {
-   if(Palette->UsageCount--<=1) delete Palette;
-   Palette = NULL;
-   }
+  {
+    if(InterlockedDecrement(&Palette->UsageCount)<=0) delete Palette;
+    Palette = NULL;
+  }
   if(NewPalette!=NULL)			// Attach raster now
-   {
-   Palette=NewPalette; Palette->UsageCount++;
-   }
+  {
+    Palette=NewPalette; InterlockedIncrement(&Palette->UsageCount);
+  }
 }
 
 
@@ -379,6 +389,17 @@ void Image::AttachProperty(PropertyItem *NewProp)
   Properties.AddProp(NewProp);
 }
 
+void Image::AttachPropList(PropertyList &PropL)
+{
+  if(Properties.isEmpty())
+  { 
+    Properties.pProperties = PropL.pProperties;
+    PropL.pProperties = NULL;
+    Properties.PropCount = PropL.PropCount;
+    PropL.PropCount = 0;
+  }
+}
+
 
 bool Image::isEmpty(void) const
 {
@@ -564,10 +585,10 @@ Raster2DAbstract *Raster2;
 	  }
         }
 
-      if(--Img->Raster->UsageCount <= 0)
-	delete Img->Raster;
+      if(InterlockedDecrement(&Img->Raster->UsageCount) <= 0)
+	  delete Img->Raster;
       Img->Raster = Raster2;
-      Img->Raster->UsageCount++;
+      InterlockedIncrement(&Img->Raster->UsageCount);
       
 
 	/* Remap a palette. */
