@@ -33,9 +33,6 @@ extern "C" {
 #include "raster.h"
 
 
-const char ExifPreffix[6] = {'E','x','i','f', '\0', '\0'};
-
-
 struct my_error_mgr {
   struct jpeg_error_mgr pub;	/* "public" fields */
 
@@ -92,7 +89,7 @@ static inline int GetProfileLength(j_decompress_ptr jpeg_info)
 static boolean ReadGenericProfile(j_decompress_ptr jpeg_info)
 {
   size_t length;
-  register size_t i;
+  size_t i;
   string profile_name;
 //  unsigned char *profile;
   int c, marker;
@@ -122,6 +119,7 @@ static boolean ReadGenericProfile(j_decompress_ptr jpeg_info)
     #define XML_MARKER  (JPEG_APP0+1)
    */
   marker = jpeg_info->unread_marker - JPEG_APP0;
+  if(marker != 1)  return TRUE;
 
 	/* Compute generic profile name. */
   profile_name.printf("APP%d",marker);
@@ -134,45 +132,67 @@ static boolean ReadGenericProfile(j_decompress_ptr jpeg_info)
 //  profile = client_data->buffer;
 
   if(length < 6) return TRUE;
-  for(i=0 ; i<6 ; i++)
+
+  i = 0;
+  c = GetCharacter(jpeg_info);
+  if(c==EOF) return TRUE;
+  if(c==ExifPreffix[0])
   {
-    c = GetCharacter(jpeg_info);
-    if(c==EOF) return TRUE;
-    if(c!=ExifPreffix[i]) return TRUE;
-  }
-
-  if(marker != 1)  return TRUE;
-
-  length -= 6;
-
-  PropertyItem *PropExif = new PropertyItem(ExifPreffix,length);
-  if(PropExif->Data==NULL)
-  {
-    delete PropExif;
-    return TRUE;
-  }
-
-  for(i=0 ; i < length ; i++)
-  {
-    c = GetCharacter(jpeg_info);
-    if(c==EOF)
+    while(++i < 6)
+    {
+      c = GetCharacter(jpeg_info);
+      if(c==EOF) return TRUE;
+      if(c!=ExifPreffix[i]) return TRUE;
+    }
+    length -= 6;
+    PropertyItem *PropExif = new PropertyItem(ExifPreffix,length);
+    if(PropExif->Data==NULL)
     {
       delete PropExif;
       return TRUE;
     }
-    ((char*)PropExif->Data)[i] = c;
+    for(i=0; i<length; i++)
+    {
+      c = GetCharacter(jpeg_info);
+      if(c==EOF)
+      {
+        delete PropExif;
+        return TRUE;
+      }
+      ((char*)PropExif->Data)[i] = c;
+    }
+    Img->AttachProperty(PropExif);
+    return TRUE;
   }
-  Img->AttachProperty(PropExif);
 
-/*
-  else if(((marker == 1) && (length > strlen(xmp_std_header)+1)) &&
-           (memcmp(profile, xmp_std_header, strlen(xmp_std_header)+1) == 0))
-  {
-	// XMP is required to fit in one 64KB chunk.  Strip off its JPEG namespace header.
-     length = strlen(xmp_std_header)+1;
-     //FormatString((char *) profile_name,"XMP");
+  if(c==xmp_std_header[0] && length>=sizeof(xmp_std_header))
+  {		// XMP is required to fit in one 64KB chunk.  Strip off its JPEG namespace header.
+    while(++i < sizeof(xmp_std_header))
+    {
+      c = GetCharacter(jpeg_info);
+      if(c==EOF) return TRUE;
+      if(c!=xmp_std_header[i]) return TRUE;
+    }
+    PropertyItem *PropXmp = new PropertyItem("XMP",length);
+    if(PropXmp->Data==NULL)
+    {
+      delete PropXmp;
+      return TRUE;
+    }
+    memcpy(PropXmp->Data,xmp_std_header,sizeof(xmp_std_header));
+    while(++i < length)
+    {
+      c = GetCharacter(jpeg_info);
+      if(c==EOF)
+      {
+        delete PropXmp;
+        return TRUE;
+      }
+      ((char*)PropXmp->Data)[i] = c;
+    }
+    Img->AttachProperty(PropXmp);
+    return TRUE;
   }
-*/
 
   //if(profile) free(profile);
   return  TRUE;
@@ -430,7 +450,7 @@ FILE * outfile;		 /* target file */
 JSAMPROW row_pointer[1]; /* pointer to JSAMPLE row[s] */
 int row_stride;		 /* physical row width in image buffer */
 
-  if(Img.Raster==NULL) return(-10);
+  if(Img.Raster==NULL) return ErrMissingRaster;
 
   /* Step 1: allocate and initialize JPEG compression object */
 
@@ -469,12 +489,18 @@ int row_stride;		 /* physical row width in image buffer */
   /* Now you can set any non-default parameters you wish to.
    * Here we just illustrate the use of quality (quantization table) scaling:   */
   i = 0;
-  Prop = Img.Properties.Find("JPG_QUALITY",i);
+  Prop = Img.Properties.Find("QUALITY",i);
   if(Prop && Prop->DataSize>=sizeof(int))
     i = *(int*)Prop->Data;
   else
     i = DEFAULT_JPG_QUALITY;
   jpeg_set_quality(&cinfo, i, TRUE /* limit to baseline-JPEG values */);
+  i = 0;
+  Prop = Img.Properties.Find("jpeg:optimize-coding",i);
+  if(Prop && Prop->DataSize>=sizeof(char))
+  {
+    cinfo.optimize_coding = *(char*)Prop->Data;
+  }
 
   /* Step 4: Start compressor */
 
@@ -516,13 +542,13 @@ int row_stride;		 /* physical row width in image buffer */
   Raster1DAbstract *RasLine;
 
   while (cinfo.next_scanline < cinfo.image_height)
-    { /* jpeg_write_scanlines expects an array of pointers to scanlines.
+  { /* jpeg_write_scanlines expects an array of pointers to scanlines.
        * Here the array is only one element long, but you could pass
        * more than one scanline at a time if that's more convenient. */
-    RasLine=Img.Raster->GetRowRaster(cinfo.next_scanline);
+    RasLine = Img.Raster->GetRowRaster(cinfo.next_scanline);
 
     switch(Img.ImageType())
-      {
+    {
       case ImagePalette:{                               //palette image
              RGBQuad RGB;
              uint8_t *ptrLine=(uint8_t *)ptrLineX;
@@ -540,11 +566,11 @@ int row_stride;		 /* physical row width in image buffer */
       default:
              RasLine->Get24BitRGB(ptrLineX);    //Gray image
              break;
-      }
+    }
        
     row_pointer[0] = (JSAMPLE *)ptrLineX;
     (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
-    }
+  }
     
   if(ptrLineX) {free(ptrLineX);ptrLineX=NULL;}    
 
@@ -562,5 +588,3 @@ int row_stride;		 /* physical row width in image buffer */
 #endif
 
 #endif
-
-
