@@ -1,5 +1,5 @@
 /******************************************************************************
- * program:     rasimg library 0.26                                           *
+ * program:     rasimg library 0.32                                           *
  * function:    Object set for handling vector images.                        *
  * modul:       vec_image.cc                                                  *
  * licency:     GPL or LGPL                                                   *
@@ -9,15 +9,12 @@
 #include <stdio.h>
 #define _USE_MATH_DEFINES
 #include <math.h>
+#include "csext.h"
 
 #include "matrix.h"
 
 #include "vecimage.h"
-
-
-#ifndef M_PI
- #define M_PI        3.14159265358979323846
-#endif
+#include "arith.h"
 
 
 static void FixPsAccent(string & s, const char *TexAccent, const char *PsAccent);
@@ -200,9 +197,9 @@ PS_State::PS_State(const PS_State &PSS)
   FontName = PSS.FontName;
   PaperBackground = PSS.PaperBackground;
   pPalette = PSS.pPalette;
-  if(pPalette!=NULL) pPalette->UsageCount++;
+  if(pPalette!=NULL) InterlockedIncrement(&pPalette->UsageCount);
   pRaster = PSS.pRaster;
-  if(pRaster!=NULL) pRaster->UsageCount++;
+  if(pRaster!=NULL) InterlockedIncrement(&pRaster->UsageCount);
 }
 
 
@@ -210,12 +207,12 @@ PS_State::~PS_State()
 {
   if(pPalette != NULL)
   {
-    if(pPalette->UsageCount--<=1) delete pPalette;
+    if(InterlockedDecrement(&pPalette->UsageCount)<=0) delete pPalette;
     pPalette = NULL;
   }
   if(pRaster != NULL)
   {
-    if(pRaster->UsageCount--<=1) delete pRaster;
+    if(InterlockedDecrement(&pRaster->UsageCount)<=0) delete pRaster;
     pRaster = NULL;
   }
 }
@@ -236,12 +233,12 @@ void PS_State::AttachPalette(APalette *NewPalette)
 {
   if(pPalette!=NULL)			// Detach previously attached raster
   {
-    if(pPalette->UsageCount--<=1) delete pPalette;
+    if(InterlockedDecrement(&pPalette->UsageCount)<=0) delete pPalette;
     pPalette = NULL;
   }
   if(NewPalette!=NULL)			// Attach raster now
   {
-    pPalette=NewPalette; pPalette->UsageCount++;
+    pPalette=NewPalette; InterlockedIncrement(&pPalette->UsageCount);
   }
 }
 
@@ -250,12 +247,12 @@ void PS_State::AttachRaster(Raster2DAbstract *NewRaster)
 {
   if(pRaster!=NULL)			// Detach previously attached raster
   {
-    if(pRaster->UsageCount--<=1) delete pRaster;
+    if(InterlockedDecrement(&pRaster->UsageCount)<=0) delete pRaster;
     pRaster = NULL;
   }
   if(NewRaster!=NULL)			// Attach raster now
   {
-    pRaster=NewRaster; pRaster->UsageCount++;
+    pRaster=NewRaster; InterlockedIncrement(&pRaster->UsageCount);
   }
 }
 
@@ -273,6 +270,7 @@ vecPen::vecPen(void)
 
 void vecPen::prepExport(PS_State *PSS) const
 {
+  //printf("Selected vecPen %d %2.2X,%2.2X,%2.2X", LineStyle, LineColor.Red, LineColor.Green, LineColor.Blue);
   if(PSS)
   {
     if(memcmp(&LineColor,&PSS->LineColor,sizeof(RGB_Record)) != 0) 
@@ -324,6 +322,7 @@ vecBrush::vecBrush(const PS_State &PSS)
 
 void vecBrush::prepExport(PS_State *PSS) const
 {
+  //printf("Selected vecBrush %d %2.2X,%2.2X,%2.2X", BrushStyle, FillColor.Red, FillColor.Green, FillColor.Blue);
   if(PSS)
   {
     if(memcmp(&FillColor,&PSS->FillColor,sizeof(RGB_Record)) != 0) 
@@ -426,7 +425,7 @@ void vecFont::prepExport(PS_State *PSS) const
 attrPalette::attrPalette(APalette *iniPalette)
 {
   pPalette = iniPalette;
-  if(pPalette!=NULL) pPalette->UsageCount++;
+  if(pPalette!=NULL) InterlockedIncrement(&pPalette->UsageCount);
 }
 
 
@@ -434,7 +433,7 @@ attrPalette::~attrPalette()
 {
   if(pPalette != NULL)
   {
-    if(pPalette->UsageCount--<=1) delete pPalette;
+    if(InterlockedDecrement(&pPalette->UsageCount)<=0) delete pPalette;
     pPalette = NULL;
   }
 }
@@ -453,9 +452,9 @@ void attrPalette::prepExport(PS_State *PSS) const
 attrRaster::attrRaster(Raster2DAbstract *iniRaster, APalette *iniPalette)
 {
   pRaster = iniRaster;
-  if(pRaster!=NULL) pRaster->UsageCount++;
+  if(pRaster!=NULL) InterlockedIncrement(&pRaster->UsageCount);
   pPalette = iniPalette;
-  if(pPalette!=NULL) pPalette->UsageCount++;
+  if(pPalette!=NULL) InterlockedIncrement(&pPalette->UsageCount);
 }
 
 
@@ -463,12 +462,12 @@ attrRaster::~attrRaster()
 {
   if(pRaster != NULL)
   {
-    if(pRaster->UsageCount--<=1) delete pRaster;
+    if(InterlockedDecrement(&pRaster->UsageCount)<=0) delete pRaster;
     pRaster = NULL;
   }
   if(pPalette != NULL)
   {
-    if(pPalette->UsageCount--<=1) delete pPalette;
+    if(InterlockedDecrement(&pPalette->UsageCount)<=0) delete pPalette;
     pPalette = NULL;
   }
 }
@@ -636,7 +635,6 @@ return pVP;
 }
 
 
-
 temp_string VectorPie::Export2EPS(PS_State *PSS) const
 {
 string str;
@@ -824,16 +822,19 @@ return pVRA;
 
 void VectorRectangleArc::Transform(const AbstractTransformXY &Tx)
 {
-float Hr1,Hr2, Vr1,Vr2;
-  Hr1 = LeftRect;	Hr2 = LeftRect + Hradius;
-  Vr1 = BottomRect;	Vr2 = BottomRect + Vradius;
+float Hr1,Hr2, Vr1,Vr2, Hr3,Vr3;
+  Hr3=Hr1 = LeftRect;	Hr2 = LeftRect + Hradius;
+  Vr3=Vr1 = BottomRect;	Vr2 = BottomRect + Vradius;
+
   VectorRectangle::Transform(Tx);  
-  Tx.ApplyTransform(Hr1, Vr1);
-  Tx.ApplyTransform(Hr2, Vr2);
-  Hradius = Hr2 - Hr1;
-  Vradius = Vr2 - Vr1;
-  if(Hradius<0) Hradius=-Hradius;
-  if(Vradius<0) Vradius=-Vradius;
+  Tx.ApplyTransform(Hr2, Vr1);			// [LeftRect+Hradius; BottomRect]
+  Tx.ApplyTransform(Hr1, Vr2);
+  Tx.ApplyTransform(Hr3, Vr3);			// Bottom and top rect could be realligned, must do a separate transform.
+
+  Hradius = sqrt(sqr(Hr3-Hr2) + sqr(Vr3-Vr1));
+  Vradius = sqrt(sqr(Hr3-Hr1) + sqr(Vr3-Vr2));
+  //if(Hradius<0) Hradius=-Hradius;		-- no need square root is always positive
+  //if(Vradius<0) Vradius=-Vradius;
 }
 
 
@@ -856,26 +857,34 @@ bool ContextSaved = false;
   //if(Tx) 
   //  ContextSaved = Tx->ApplyTransform(PSData);
   
-  float Y_ll = BottomRect;
-  float Y_ur = TopRect;
+  float HRadiusFix = Hradius;
+  float VRadiusFix = Vradius;
 
-  if(fabs(Hradius-Vradius) > 1e-5)
+  float Y_ur = fabs(RightRect-LeftRect)/2;
+  if(Hradius > Y_ur) HRadiusFix=Y_ur;
+  Y_ur = fabs(TopRect-BottomRect)/2;
+  if(Vradius > Y_ur) VRadiusFix=Y_ur;
+
+  float Y_ll = BottomRect;
+  Y_ur = TopRect;
+
+  if(fabs(HRadiusFix-VRadiusFix) > 1e-5)
   {	//Horizontal and Vertical radiuses are different - scale canvas
-    const float Ty = Vradius/Hradius;
+    const float Ty = VRadiusFix/HRadiusFix;
     if(!ContextSaved) PSData+="\ngsave";
-    PSData.cat_printf("\n0 %2.2f translate\n1 %2.3f scale", Y_ll, 1.0/Ty);
+    PSData.cat_printf("\n0 %2.2f translate\n1 %2.3f scale", Y_ll, Ty);
     Y_ur -= Y_ll;
     Y_ll = 0;
-    Y_ur *= Ty;
+    Y_ur /= Ty;
     ContextSaved = true;
  }
 
  PSData += "\nnewpath";
- PSData.cat_printf("\n%2.2f %2.2f moveto", RightRect-Hradius, Y_ll);
- PSData.cat_printf("\n%2.2f %2.2f %2.2f 270 0 arc", RightRect-Hradius, Y_ll+Hradius, Hradius);
- PSData.cat_printf("\n%2.2f %2.2f %2.2f 0 90 arc", RightRect-Hradius, Y_ur-Hradius, Hradius);
- PSData.cat_printf("\n%2.2f %2.2f %2.2f 90 180 arc", LeftRect+Hradius, Y_ur-Hradius, Hradius);
- PSData.cat_printf("\n%2.2f %2.2f %2.2f 180 270 arc", LeftRect+Hradius, Y_ll+Hradius, Hradius);
+ PSData.cat_printf("\n%2.2f %2.2f moveto", RightRect-HRadiusFix, Y_ll);
+ PSData.cat_printf("\n%2.2f %2.2f %2.2f 270 0 arc", RightRect-HRadiusFix, Y_ll+HRadiusFix, HRadiusFix);
+ PSData.cat_printf("\n%2.2f %2.2f %2.2f 0 90 arc", RightRect-HRadiusFix, Y_ur-HRadiusFix, HRadiusFix);
+ PSData.cat_printf("\n%2.2f %2.2f %2.2f 90 180 arc", LeftRect+HRadiusFix, Y_ur-HRadiusFix, HRadiusFix);
+ PSData.cat_printf("\n%2.2f %2.2f %2.2f 180 270 arc", LeftRect+HRadiusFix, Y_ll+HRadiusFix, HRadiusFix);
 
  PSData+="\nclosepath";
  FillObjectPS(RightRect, LeftRect, Y_ll, Y_ur,PSData,PSS);
@@ -909,6 +918,7 @@ string str;
       vecBrush::prepExport(PSS);
       vecPen::prepExport(PSS);
       PS_Attr(str,PSS);
+/* This is not working. Separate polygons are not clipping.
       if(PSS->PolyFillMode==1 && UpperCont!=NULL && CountPoints>=3)
       {
         unsigned IntersectCount=0;
@@ -925,6 +935,7 @@ string str;
           PSS->FirstTimeFix = true;	// Flip foreground to background.
         }
       }
+*/
     }
   }
 
@@ -969,7 +980,7 @@ string str;
 void VectorPolygon::Dump2File(FILE *F) const
 {
 const float *pEnd = Points + 2*CountPoints;
-  fputs("VectorPolygon{",F);
+  fprintf(F,"VectorPolygon{Brush=%2.2X,%2.2X,%2.2X",FillColor.Red,FillColor.Green,FillColor.Blue);
   for(float *p=Points; p<pEnd; p+=2)
   {
     fprintf(F,"[%2.2f;%2.2f]", p[0], p[1]);
@@ -1752,7 +1763,7 @@ TextContainer::TextContainer()
   TextObjects = 0;
   FontOrientation = 0;
   RotCenterX = RotCenterY = 0;
-  TextAllign = 0;
+  TextAlign = 0;
   MixMode = 0;
 }
 
@@ -1784,7 +1795,7 @@ VectorObject *TextContainer::Duplicate(void) const
   pTC->FontOrientation = FontOrientation;	///< Font rotation [deg]
   pTC->RotCenterX = RotCenterX;
   pTC->RotCenterY = RotCenterY;
-  pTC->TextAllign = TextAllign;
+  pTC->TextAlign = TextAlign;
   pTC->MixMode = MixMode;
   if(Text!=NULL)
   {
@@ -1932,7 +1943,7 @@ string Override;
         FixPsAccent(Text[i]->contents,"\\\"{","(\\310)");	// umlaut
         FixPsAccent2(Text[i]->contents,"\\accent39","( \\47)"); //insert space before apostrophe \\47
         
-        if(i>0) AppendShow(str,strLine,TextAllign,Override);
+        if(i>0) AppendShow(str,strLine,TextAlign,Override);
 	
         if(PSS!=NULL)
         {
@@ -1978,7 +1989,7 @@ string Override;
 	  ScanS = ScanS2 + 1;
 
 	  CurPosY -= 1.01f * Text[i]->size*2.66f;
-	  AppendShow(str,strLine,TextAllign,Override);
+	  AppendShow(str,strLine,TextAlign,Override);
 	  str.cat_printf(" newpath %2.2f %2.2f moveto", CurPosX, CurPosY);
 	}
 
@@ -1990,7 +2001,7 @@ string Override;
         }
       }      
     }
-    AppendShow(str,strLine,TextAllign,Override);
+    AppendShow(str,strLine,TextAlign,Override);
     if(TextRotated)
     {    
       str += "\ngrestore";
