@@ -1,5 +1,5 @@
 /***************************************************************************
- * Unit:    rasimg library 0.26                                            *
+ * Unit:    rasimg library 0.32                                            *
  * Purpose: Prototypes for general manipulation vith vector images         *
  * Modul:   vecimage.h                                                     *
  * Licency: GPL or LGPL                                                    *
@@ -112,7 +112,7 @@ typedef struct
 	uint8_t LineCap;		///< 0-butt caps, 1-round caps, 2-extended butt caps.
 	uint8_t LineJoin;		///< 0-miter join, 1-round join, 2-bevel join.
 	uint8_t PolyFillMode;		///< 0-standard; 1-alternating
-	uint8_t TextAllign;		///< 0-TA_LEFT; 2-TA_RIGHT; 6-TA_CENTER
+	uint8_t TextAlign;		///< 0-TA_LEFT; 2-TA_RIGHT; 6-TA_CENTER
 
 	RGB_Record FillColor;	
 	RGB_Record FillBackground;
@@ -230,7 +230,7 @@ class vecBrush: public VectorAttribute
 public:
   RGB_Record FillColor;
   RGB_Record FillBackground;
-  unsigned char BrushStyle;
+  unsigned char BrushStyle;		///< Fill pattern.
 
   vecBrush() {BrushStyle=1;}
   vecBrush(const PS_State &PSS);
@@ -543,7 +543,7 @@ public:
   float PosX, PosY;		///< Tex start point.
   float FontOrientation;	///< Font rotation [deg]
   float RotCenterX, RotCenterY;
-  uint8_t TextAllign;
+  uint8_t TextAlign;
   uint8_t MixMode;		///< 1 Transparent; 2 Opaque
 
   TextContainer();
