@@ -22,7 +22,7 @@
  #include <unistd.h>
 #endif
 
-#include"images/raster.h"
+#include "raster.h"
 
 #include"wp2latex.h"
 
