@@ -8,6 +8,7 @@
  ******************************************************************************/
 // https://www.w3schools.com/graphics/svg_intro.asp
 // https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#path_commands
+// http://xahlee.info/js/svg_path_ellipse_arc.html
 #include<stdio.h>
 #include<string.h>
 #include<stdlib.h>
@@ -23,7 +24,7 @@
 #include "dbllist.h"
 #include "matrix.h"
 
-#include "images/raster.h"
+#include "raster.h"
 #include "images/vecimage.h"
 #include "images.h"
 
@@ -33,10 +34,7 @@
 
 #include "wmfsupp.h"
 
-
-#ifndef M_PI
- #define M_PI        3.14159265358979323846
-#endif
+#include "arith.h"
 
 
 extern list HTMLChars;
@@ -172,6 +170,12 @@ bool ReadFloat(const char **pstr, float &f)
 
   if(!isdigit(*str) && *str!='.' && *str!='-')
   {
+    if(!strnicmp(str,"nan",3))
+    {
+      *pstr = str+3;
+      MakeNaN(&f);
+      return true;
+    }
     *pstr = str;
     f = 0;
     return false;
@@ -193,7 +197,7 @@ return true;
 }
 
 
-inline float DEG2RAD(const float x) {return((x)*(float)M_PI/180.0f);}
+//inline float DEG2RAD(const float x) {return((x)*(float)M_PI/180.0f);}
 
 
 ////////////////////////////////////////////////////
@@ -219,6 +223,7 @@ typedef enum
 	SVG_POLYLINE = 134,
 	SVG_POLYGON = 135,
 	SVG_PATH = 136,
+	SVG_A,
 	SVG_G,
 	SVG_TEXT,
 	SVG_DEFS,
@@ -241,6 +246,8 @@ protected:
      int DObjectCount;
      ForwardHolderBase **DelayedObjects;		///< List of mostly empty objects that are simultaneosly stored in the main VectorObject list.
      int DelayedObjCount;
+     CpTranslator *PsNativeCP;
+     CpTranslator *PsNativeSym;
 
      //RGB_Record DefaultLineColor;
      //RGB_Record DefaultFillColor;
@@ -267,12 +274,14 @@ public:
      void EraseObjCache(void);
      DefinedObject *GetDefine(const char *DefId);
      void AddDelayedObject(ForwardHolderBase *pFwdObj);
+     TextContainer *FlushText(float x, float & y, int LineLen, TextContainer *pTextCont);
 
      virtual int Convert_first_pass(void);
      void ProcessKeySVG(void);
      void ProcessImage(void);
      void SolveDelayedObjs(void);
 
+     void Anchor(void);
      void Circle(void);
      void Defs(void);
      void Ellipse(void);
@@ -303,14 +312,16 @@ ImplementReader(SVG,".SVG")
    LoadPictureSVG,
    NULL);
 
-#define SVGVersion "0.10"
+#define SVGVersion "0.14"
 
 
 TconvertedPass1_SVG::TconvertedPass1_SVG(void): DelayedObjects(NULL), objs(NULL) 
 {
   PSS.LineStyle = 0;
-  PSS.FillPattern = FILL_NONE;
+  PSS.FillPattern = FILL_SOLID;		// Default fill is a solid black.
   DelayedObjCount = DObjectCount = 0;
+  PsNativeCP = GetTranslator("internalTOcp1276");
+  PsNativeSym = GetTranslator("internalTOsymbol");
 }
 
 
@@ -426,7 +437,7 @@ typedef struct
 } SVG_color;
 
 // https://www.w3.org/TR/css-color-3/#html4
-static SVG_color SVG_colors[] =
+static const SVG_color SVG_colors[] =
 {
  "black", {0,0,0},
  "silver", {192,192,192},
@@ -447,44 +458,11 @@ static SVG_color SVG_colors[] =
 };
 
 
-/** Decode color.
- * @return -1 on failure; 0 when no color specified; 1 Color is OK; 2 reference to a style */
-int TconvertedPass1_SVG::GetColor(RGB_Record &LineColor, const char *ClrType)
-{
-int i;
-
-  if(ClrType==NULL || *ClrType==0) return -1;
-  if((i=TAG_Args IN ClrType)<0) return -1;
-  const char * const Payload = TAG_Args.Member(i,1);
-  if(Payload==NULL) return -1;
-
-  if(!strcmp(Payload, "none")) return 0;
- 
-  if(!strncmp(Payload,"url(#",5)) return 2;
-
-  if(Payload[0]=='#' && isxdigit(Payload[1]))
-  {
-    const uint32_t val = strtoul(Payload+1, NULL, 16);
-    LineColor.Blue = val & 0xFF;
-    LineColor.Green = (val>>8) & 0xFF;
-    LineColor.Red = (val>>16) & 0xFF;
-    return 1;
-  }
-  for(i=0; i<sizeof(SVG_colors)/sizeof(SVG_color);i++)
-    if(!strcmp(Payload, SVG_colors[i].desc))
-    {
-      LineColor = SVG_colors[i].color;
-      return 1;
-    }
-
-return -1;
-}
-
-
 /// Extract a color from style string item.
-bool GetStyleColor(RGB_Record *pColor, const char *ClrText)
+/// @return	-1 on failure; 0 when no color specified; 1 Color is OK; 2 reference to a style.
+int GetStyleColor(RGB_Record *pColor, const char *ClrText)
 {
-  if(ClrText==NULL) return false;
+  if(ClrText==NULL) return -1;
 
   while(isspace(*ClrText)) ClrText++;
   if(!strncmp(ClrText,"rgb",3))
@@ -505,11 +483,11 @@ bool GetStyleColor(RGB_Record *pColor, const char *ClrText)
         {
           ClrText++;
           pColor->Blue = atoi(ClrText);
-          return true;
+          return 1;
         }
       }
     }
-    return false;
+    return -1;
   }
 
   if(*ClrText=='#')
@@ -517,11 +495,26 @@ bool GetStyleColor(RGB_Record *pColor, const char *ClrText)
     ClrText++;
     while(isspace(*ClrText)) ClrText++;
     if(*ClrText==0) return false;
-    unsigned number = strtoul(ClrText, NULL, 16);
-    pColor->Blue = number & 0xFF;
-    pColor->Green = (number>>8) & 0xFF;
-    pColor->Red = (number>>16) & 0xFF;
-    return true;
+    const unsigned number = strtoul(ClrText, NULL, 16);
+    int letters = 0;
+    while(isxdigit(*ClrText) && letters<4)
+    {
+      letters++;
+      ClrText++;
+    }
+    if(letters<=3)
+    {
+      pColor->Blue = 0x11 * (number & 0xF);
+      pColor->Green = 0x11 * ((number>>4) & 0xF);
+      pColor->Red = 0x11 * ((number>>8) & 0xF);
+    }
+    else
+    {
+      pColor->Blue = number & 0xFF;
+      pColor->Green = (number>>8) & 0xFF;
+      pColor->Red = (number>>16) & 0xFF;
+    }
+    return 1;
   }
 
 	// Look into a color dictionary.
@@ -533,15 +526,33 @@ bool GetStyleColor(RGB_Record *pColor, const char *ClrText)
   }
   if(Color.length() > 0)
   {
+    if(Color=="none") return 0;
     for(int i=0; i<sizeof(SVG_colors)/sizeof(SVG_color);i++)
       if(!strcmp(Color(), SVG_colors[i].desc))
       {
         *pColor = SVG_colors[i].color;
-        return true;
+        return 1;
       }
   }
 
-  return false;
+  return -1;
+}
+
+
+/// Decode color.
+/// @return -1 on failure; 0 when no color specified; 1 Color is OK; 2 reference to a style.
+int TconvertedPass1_SVG::GetColor(RGB_Record &LineColor, const char *ClrType)
+{
+int i;
+
+  if(ClrType==NULL || *ClrType==0) return -1;
+  if((i=TAG_Args IN ClrType)<0) return -1;
+  const char * const Payload = TAG_Args.Member(i,1);
+  if(Payload==NULL) return -1;
+ 
+  if(!strncmp(Payload,"url(#",5)) return 2;
+
+  return(GetStyleColor(&LineColor,Payload));
 }
 
 
@@ -591,7 +602,7 @@ const char *str;
     if(str)
     {
       str += 7;
-      if(GetStyleColor(&pPen->LineColor,str))
+      if(GetStyleColor(&pPen->LineColor,str)>0)
           pPen->LineStyle = 1;
     }
   }
@@ -601,8 +612,12 @@ const char *str;
     if(str)
     {
       str += 5;
-      if(GetStyleColor(&pBrush->FillColor,str))
-          pBrush->BrushStyle = FILL_SOLID;
+      switch(GetStyleColor(&pBrush->FillColor,str))
+      {
+        case 0: pBrush->BrushStyle=FILL_NONE; break;
+        case 1: pBrush->BrushStyle=FILL_SOLID; break;
+        default: break;
+      }
     }
   }
 }
@@ -751,12 +766,25 @@ int i;
         if(*StrTransform==',') 
         {
           StrTransform++;
-          while(isspace(*StrTransform)) StrTransform++;
         }
-        if(ReadFloat(&StrTransform, Tx(3,1)))
+        while(isspace(*StrTransform)) StrTransform++;
+        if(*StrTransform==')')		// only one argument for 2 directions
+        {
+          Tx(3,1) = Tx(3,0);
+          pTransformSVG->CTM *= Tx;
+        }
+        else if(ReadFloat(&StrTransform, Tx(3,1)))
         {
           pTransformSVG->CTM *= Tx;
         }
+        else
+        {
+          if(err != NULL)
+          {
+            perc.Hide();
+            fprintf(err,_("\nError: Unexpected content in translate transform command: \"%s\"."),StrTransform);
+          }
+        }
       }
     } else if(!strncmp("scaleX",StrTransform,6))
     {
@@ -792,16 +820,29 @@ int i;
         float_matrix Tx(4,4,(float)0);
         if(ReadFloat(&StrTransform, Tx(0,0)))
         {
-           Tx(2,2) = Tx(3,3) = 1;
-           if(*StrTransform==',') 
-           {
-             StrTransform++;
-             while(isspace(*StrTransform)) StrTransform++;
-           }
-           if(ReadFloat(&StrTransform, Tx(1,1)))
-           {
-             pTransformSVG->CTM *= Tx;
-           }
+          Tx(2,2) = Tx(3,3) = 1;
+          if(*StrTransform==',') 
+          {
+            StrTransform++;
+          }
+          while(isspace(*StrTransform)) StrTransform++;
+          if(*StrTransform==')')		// only one argument for 2 directions
+          {
+            Tx(1,1) = Tx(0,0);
+            pTransformSVG->CTM *= Tx;
+          }
+          else if(ReadFloat(&StrTransform, Tx(1,1)))	// different scales in both spatial directions
+          {
+            pTransformSVG->CTM *= Tx;
+          }
+          else
+          {
+            if(err != NULL)
+            {
+              perc.Hide();
+              fprintf(err,_("\nError: Unexpected content in translate scale command: \"%s\"."),StrTransform);
+            }
+          }
         }
       }
     } else if(!strncmp("rotate",StrTransform,6))
@@ -847,12 +888,21 @@ int i;
       if(*StrTransform==',') StrTransform++;
       Result &= ReadFloat(&StrTransform, Tx(3,1));
 
+#ifdef _DEBUG
+/*
+      for(int y=0; y<4; y++)
+        for(int x=0; x<4; x++) fprintf(log," %f",Tx(x,y));
+      fprintf(log,"\n");
+*/
+#endif
+
       if(Result)
       {
         pTransformSVG->CTM *= Tx;
       }
     } else
     {			// No keyword recognised; skip one character.
+      fprintf(err,_("\nError: unrecognised SVG transform keyword: \"%s\"."), StrTransform);
       StrTransform++;
       continue;
     }
@@ -985,14 +1035,15 @@ ExitErr:
 }
 
 
-void TconvertedPass1_SVG::Group(void)
+void TconvertedPass1_SVG::Anchor(void)
 {
 #ifdef DEBUG
-  fprintf(log,"\n#TconvertedPass1_SVG::Group ");fflush(log);
+  fprintf(log,"\n#TconvertedPass1_SVG::Anchor ");fflush(log);
 #endif
 VectorList VectListIn;
 FloatBBox bbxBk = bbx;
 const RGB_Record LineColorBK = PSS.LineColor;
+const RGB_Record FillColorBK = PSS.FillColor;
 const unsigned char LineCapBK = PSS.LineCap;
 const unsigned char LineJoinBK = PSS.LineJoin;
 const float FontSizeBk = PSS.FontSize;
@@ -1000,6 +1051,7 @@ const float FontSizeBk = PSS.FontSize;
   GetLineCap(PSS.LineCap);
   GetLineJoin(PSS.LineJoin);
   GetColor(PSS.LineColor);
+  GetColor(PSS.FillColor,FILL_COLOR);
   AbstractTransformXY *pTRx = getTransform();
   if(!GetProp("font-size",PSS.FontSize))
     PSS.FontSize = FontSizeBk;
@@ -1012,7 +1064,7 @@ const float FontSizeBk = PSS.FontSize;
   while(!feof(wpd))
   {
     ReadXMLTag(false);
-    if(by==XML_closetag && TAG=="</g>")
+    if(by==XML_closetag && TAG=="</a>")
         break;
     ProcessKeySVG();
   }
@@ -1049,6 +1101,80 @@ if(bbx.MaxX>=bbx.MinX && bbx.MaxY>=bbx.MinY)
   PSS.LineJoin = LineJoinBK;
   PSS.LineColor = LineColorBK;
   PSS.FontSize = FontSizeBk;
+  PSS.FillColor = FillColorBK;
+
+strcpy(ObjType,"A");
+}
+
+
+void TconvertedPass1_SVG::Group(void)
+{
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_SVG::Group ");fflush(log);
+#endif
+VectorList VectListIn;
+FloatBBox bbxBk = bbx;
+const RGB_Record LineColorBK = PSS.LineColor;
+const RGB_Record FillColorBK = PSS.FillColor;
+const unsigned char LineCapBK = PSS.LineCap;
+const unsigned char LineJoinBK = PSS.LineJoin;
+const float FontSizeBk = PSS.FontSize;
+
+  GetLineCap(PSS.LineCap);
+  GetLineJoin(PSS.LineJoin);
+  GetColor(PSS.LineColor);
+  GetColor(PSS.FillColor,FILL_COLOR);
+  AbstractTransformXY *pTRx = getTransform();
+
+  if(!GetProp("font-size",PSS.FontSize))
+    PSS.FontSize = FontSizeBk;
+  else
+    PSS.FontSize /= 2.66;		// [pt] --> [mm]
+
+  InitBBox(bbx);
+  VectList.Swap(VectListIn);
+  recursion++;
+  while(!feof(wpd))
+  {
+    ReadXMLTag(false);
+    if(by==XML_closetag && TAG=="</g>")
+        break;
+    ProcessKeySVG();
+  }
+  recursion--;
+  VectList.Swap(VectListIn);
+
+  if(VectListIn.VectorObjects > 0)
+  {		// Bounding box is in PS units.
+    if(pTRx != NULL)
+    {
+#ifdef _DEBUG
+
+      if(bbx.MaxX>=bbx.MinX && bbx.MaxY>=bbx.MinY)
+      {
+        VectorRectangle *pVR=new VectorRectangle(bbx.MinY, bbx.MaxY, bbx.MinX, bbx.MaxX);
+        pVR->LineColor.Red = 128;
+        pVR->LineColor.Green = 128;
+        pVR->BrushStyle = 0;
+        VectListIn.AddObject(pVR);
+      }
+
+#endif
+      VectListIn.Transform(*pTRx);
+    }
+    VectList.Append(VectListIn);
+  }
+if(bbx.MaxX>=bbx.MinX && bbx.MaxY>=bbx.MinY)
+        UpdateBBox(bbxBk,pTRx, bbx);
+  bbx = bbxBk;
+
+  if(pTRx != NULL) {delete(pTRx); pTRx=NULL;}
+
+  PSS.LineCap = LineCapBK;
+  PSS.LineJoin = LineJoinBK;
+  PSS.LineColor = LineColorBK;
+  PSS.FontSize = FontSizeBk;
+  PSS.FillColor = FillColorBK;
 
 strcpy(ObjType,"Group");
 }
@@ -1126,69 +1252,216 @@ string key, commands;
 }
 
 
+
+TextContainer *TconvertedPass1_SVG::FlushText(float x, float & y, int LineLen, TextContainer *pTextCont)
+{
+  if(pTextCont==NULL) return NULL;
+  if(pTextCont->isEmpty()) return pTextCont;
+  
+  pTextCont->PosX = x;
+  pTextCont->PosY = y;
+  //pTextCont->AddText(temp_string(txt),PSS);
+  VectList.AddObject(pTextCont);
+  float FontSize = (PSS.FontSize <= 0) ? 0.25f : PSS.FontSize;
+  UpdateBBox(bbx,0, x,y, LineLen*mm2PSu(FontSize), -1.1f*mm2PSu(FontSize));
+  y += 1.1f*mm2PSu(FontSize);
+return new TextContainer;
+}
+
+
 void TconvertedPass1_SVG::Text(void)
 {
 #ifdef DEBUG
   fprintf(log,"\n#TconvertedPass1_SVG::Text ");fflush(log);
 #endif
-string txt;
 float x, y;
 int LineLen = 0;
+int i;
 const RGB_Record BkTextColor = PSS.TextColor;
 const float FontSizeBk = PSS.FontSize;
+bool SpaceAdded = false;
 
   if(!GetProp("x",x)) x=0;
   if(!GetProp("y",y)) y=0;
+
+  if((i=TAG_Args IN "style")>=0)
+  {
+    const char *arg = TAG_Args.Member(i,1);
+    if(arg)
+    {
+      const char *str = strstr(arg,"font-size:");
+      if(str)
+      {
+        str += 10;
+        PSS.FontSize = atof(str)/2.66;		// [pt] --> [mm]
+      }
+      str = strstr(arg,"fill:");
+      if(str)
+      {
+        GetStyleColor(&PSS.TextColor,str+5);
+      }
+      //GetStyle(pPen, pBrush, TAG_Args.Member(i,1));
+    }
+  }
+
   GetColor(PSS.TextColor,FILL_COLOR);
-  if(!GetProp("font-size",PSS.FontSize))
-    PSS.FontSize = FontSizeBk;
-  else
+  if(GetProp("font-size",PSS.FontSize))
+  {
     PSS.FontSize /= 2.66;		// [pt] --> [mm]
+  }
 
   recursion++;
+  TextContainer *pTextCont = new TextContainer;  
   while(!feof(wpd))
   {
     ReadXMLTag(false);
     if(by==XML_closetag && TAG=="</text>")
         break;
-    if(by==XML_char)
+
+    if(by==XML_tag && TAG=="<tspan>")
     {
-      if(!txt.isEmpty())
+      if(!pTextCont->isEmpty())
       {
-        if(txt[txt.length()-1] == '\n')
-        {
-          txt[txt.length()-1] = ' ';
-          if(subby=='\r') continue;
-        }
-        if(txt[txt.length()-1] == '\r')
-        {
-          txt[txt.length()-1] = ' ';
-          if(subby=='\n') continue;
-        }
+        pTextCont = FlushText(x,y,LineLen,pTextCont);
+        LineLen = 0;
       }
 
-      txt += subby;
-      LineLen++;
-      continue;
+      recursion++;
+      const float FontSizeBkSpan = PSS.FontSize;
+      const RGB_Record TextColorBk = PSS.TextColor;
+      const float BkX = x;
+      const float BkY = y;
+
+      GetProp("x",x);
+      GetProp("y",y);
+      if((i=TAG_Args IN "style")>=0)
+      {
+        const char *arg = TAG_Args.Member(i,1);
+        if(arg)
+        {
+          const char *str = strstr(arg,"font-size:");
+          if(str)
+          {
+            str += 10;
+            PSS.FontSize = atof(str)/2.66;		// [pt] --> [mm]
+          }
+          str = strstr(arg,"fill:");
+          if(str)
+          {
+            GetStyleColor(&PSS.TextColor,str+5);
+          }
+          //GetStyle(pPen, pBrush, TAG_Args.Member(i,1));
+        }
+      }
+      GetColor(PSS.TextColor,FILL_COLOR);
+      if(GetProp("font-size",PSS.FontSize))
+      {
+        PSS.FontSize /= 2.66;		// [pt] --> [mm]
+      }
+
+      while(!feof(wpd))
+      {
+        ReadXMLTag(false);
+        if(by==XML_closetag && TAG=="</tspan>")
+            break;
+        switch(by)
+        {
+          case XML_char:
+              if(log!=NULL) fputc(subby,log);
+              if(subby=='\r' || subby=='\n')
+              {
+                if(!pTextCont->isEmpty()) continue;
+                if(!SpaceAdded)
+                  AddCharacterToContainer(pTextCont, ' ', PsNativeCP, PsNativeSym, this, PSS);
+                SpaceAdded = true;
+                continue;
+              }
+              SpaceAdded = subby==' ';
+              AddCharacterToContainer(pTextCont, subby, PsNativeCP, PsNativeSym, this, PSS);
+              LineLen++;
+              continue;
+
+          case XML_extchar:
+	      if(!TAG.isEmpty())
+              {
+                if(log!=NULL) fputs(TAG(),log);
+                if(TAG[0]=='&' && TAG[TAG.length()-1]==';')
+                  TAG = copy(TAG,1,TAG.length()-2);
+                int i = (TAG() IN HTMLChars);
+                if(i>0 && ConvertHTML!=NULL)
+		{
+                  const uint16_t WchI = (*ConvertHTML)[i-1];
+                  AddCharacterToContainer(pTextCont, WchI, PsNativeCP, PsNativeSym, this, PSS);
+                  SpaceAdded = false;
+                  LineLen++;
+                }
+              }
+              continue;
+        }    
+        ProcessKeySVG();
+      }
+
+      if(!pTextCont->isEmpty())
+      {
+        pTextCont = FlushText(x,y,LineLen,pTextCont);
+        LineLen = 0;
+      }
+
+      PSS.FontSize = FontSizeBkSpan;
+      PSS.TextColor = TextColorBk;
+      x = BkX;
+      y = BkY;
+      recursion--;
+    }
+
+    switch(by)
+    {
+      case XML_char:
+          if(subby=='\r' || subby=='\n')
+          {
+            if(!pTextCont->isEmpty()) continue;
+            if(!SpaceAdded)
+                AddCharacterToContainer(pTextCont, ' ', PsNativeCP, PsNativeSym, this, PSS);
+            SpaceAdded = true;
+            continue;
+          }
+          AddCharacterToContainer(pTextCont, subby, PsNativeCP, PsNativeSym, this, PSS);
+          SpaceAdded = subby==' ';
+          LineLen++;
+          continue;
+
+      case XML_extchar:
+          if(!TAG.isEmpty())
+	  {
+            if(log!=NULL) fputs(TAG(),log);
+            if(TAG[0]=='&' && TAG[TAG.length()-1]==';')
+                TAG = copy(TAG,1,TAG.length()-2);
+            int i = (TAG() IN HTMLChars);
+            if(i>0 && ConvertHTML!=NULL)
+	    {
+              const uint16_t WchI = (*ConvertHTML)[i-1];
+              AddCharacterToContainer(pTextCont, WchI, PsNativeCP, PsNativeSym, this, PSS);
+              SpaceAdded = false;
+              LineLen++;
+            }
+          }
+          continue;
     }
     ProcessKeySVG();
   }
   recursion--;
 
-  txt = replacesubstring(txt,"\n"," ");
-  txt = replacesubstring(txt,"\r"," ");
-  LineLen = txt.length();
+  //if(log!=NULL && !txt.isEmpty())
+  //    fputs(txt(),log);
+  //txt = replacesubstring(txt,"\n"," ");
+  //txt = replacesubstring(txt,"\r"," ");
+  //LineLen = txt.length();
 
-  if(!txt.isEmpty())
+  if(!pTextCont->isEmpty())
   {
-    TextContainer *pTextCont = new TextContainer;
-    pTextCont->PosX = x;
-    pTextCont->PosY = y;
-    pTextCont->AddText(temp_string(txt),PSS);
-    VectList.AddObject(pTextCont);
-    float FontSize = (PSS.FontSize <= 0) ? 0.25f : PSS.FontSize;
-    UpdateBBox(bbx,0, x,y, LineLen*mm2PSu(FontSize), -1.1f*mm2PSu(FontSize));
+    pTextCont = FlushText(x,y,LineLen,pTextCont);
   }
+  delete pTextCont;
 
   PSS.TextColor = BkTextColor;
   PSS.FontSize = FontSizeBk;
@@ -1402,7 +1675,7 @@ float x, y, Width, Height;
     }
   }
 
-  if((i=TAG_Args IN "xlink:href") > 0)
+  if((i=TAG_Args IN "xlink:href") >= 0)
   {
     char *Mime = TAG_Args.Member(i,1);
     char *Contents = Mime;
@@ -1457,16 +1730,41 @@ float x, y, Width, Height;
     }
   }
 
-  Image Img2(LoadPicture(NewFileName()));
-  if(!Img2.isEmpty())
+  if((i=TAG_Args IN "href") >= 0)
   {
-    if(Img2.Raster!=NULL)
+    const char *Contents = TAG_Args.Member(i,1);
+    if(Contents!=NULL && !strncmp(Contents,"data:",5))
     {
-       VectorRaster *VecR = new VectorRaster(y, y+Height, x+Width, x);
-       VecR->AttachRaster(Img2.Raster);
-       VecR->AttachPalette(Img2.Palette);
-       VecR->CalcBoundingBox(bbx);
-       VectList.AddObject(VecR);
+      for(i=0; i<sizeof(MimeList)/sizeof(TMimeItem); i++)
+      {
+        if(!strncmp(Contents,MimeList[i].MimeType,MimeList[i].TypeLen))
+        {
+          Contents += MimeList[i].TypeLen;
+          if(*Contents==',') Contents++;
+          NewFileName = MergePaths(OutputDir,RelativeFigDir) + GetSomeImgName(MimeList[i].ImageExt);
+          FILE *F=fopen(NewFileName(),"wb");
+          if(F==NULL) {NewFileName.erase();break;}
+          base64_decode(Contents, F);
+          fclose(F);
+          break;
+        }
+      }
+    }
+  }
+
+  if(!NewFileName.isEmpty())
+  {
+    Image Img2(LoadPicture(NewFileName()));
+    if(!Img2.isEmpty())
+    {
+      if(Img2.Raster!=NULL)
+      {
+         VectorRaster *VecR = new VectorRaster(y, y+Height, x+Width, x);
+         VecR->AttachRaster(Img2.Raster);
+         VecR->AttachPalette(Img2.Palette);
+         VecR->CalcBoundingBox(bbx);
+         VectList.AddObject(VecR);
+      }
     }
   }
 
@@ -1481,7 +1779,7 @@ return;
 
 float SolveQuadraticPos(const float anch, const float s)
 {
-  return (s-anch) / sqrt(2.0) + anch;
+  return (s-anch) / sqrtf(2.0) + anch;
 }
 
 
@@ -1527,6 +1825,7 @@ void TconvertedPass1_SVG::SolveExtProperty(const char *PropName, vecPen *pVvecPe
 }
 
 
+// https://developer.mozilla.org/en-US/docs/Web/SVG/Tutorial/Paths
 void TconvertedPass1_SVG::Path(void)
 {
 #ifdef DEBUG
@@ -1566,6 +1865,96 @@ VectorList VectListIn;
         LastCommand = *StrMovements++;
     switch(LastCommand)
     {
+      case 'A':		// A rx ry x-axis-rotation large-arc-flag sweep-flag x y  see https://www.nan.fyi/svg-paths/arcs
+        if(n>0) SegmentCount++;
+        if(isCurve)
+          ClosePathCurve(&Points,n,false,Tx);
+        else
+          ClosePathLine(&Points,n,false,Tx);
+        {
+          float rx, ry, rotation, x, y;
+          char large_arc, sweep;
+          ReadFloat(&StrMovements, rx);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, ry);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, rotation);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, val);
+          if(*StrMovements==',') StrMovements++;
+	  large_arc = (val==0) ? 0 : 1;
+          ReadFloat(&StrMovements, val);
+          if(*StrMovements==',') StrMovements++;
+	  sweep = (val==0) ? 0 : 1;
+          ReadFloat(&StrMovements, x);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, y);
+
+          if(Points==NULL)
+          {
+            Points = (float*)malloc(4*sizeof(float));
+            if(!Points) goto ExitErr;
+            Points[0]=PositionX; Points[1]=PositionY;
+            Points[2]=x;         Points[3]=y;
+            n = 2;
+            ClosePathLine(&Points,n,false,Tx);
+          }
+          else
+          {
+	    PositionX = x;
+	    PositionY = y;
+          }
+          //VectorEllipse *pVectCirc = new VectorEllipse(cy-r, cy+r, cx+r, cx-r);
+        }
+        if(err!=NULL)
+            fprintf(err,_("\nError: Path command 'A' is only partially supported!"));
+	break;
+
+      case 'a':		// A rx ry x-axis-rotation large-arc-flag sweep-flag dx dy  see https://www.nan.fyi/svg-paths/arcs
+        if(n>0) SegmentCount++;
+        if(isCurve)
+          ClosePathCurve(&Points,n,false,Tx);
+        else
+          ClosePathLine(&Points,n,false,Tx);
+        {
+          float rx, ry, rotation, dx, dy;
+          char large_arc, sweep;
+          ReadFloat(&StrMovements, rx);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, ry);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, rotation);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, val);
+          if(*StrMovements==',') StrMovements++;
+	  large_arc = (val==0) ? 0 : 1;
+          ReadFloat(&StrMovements, val);
+          if(*StrMovements==',') StrMovements++;
+	  sweep = (val==0) ? 0 : 1;
+          ReadFloat(&StrMovements, dx);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, dy);
+
+          if(Points==NULL)
+          {
+            Points = (float*)malloc(4*sizeof(float));
+            if(!Points) goto ExitErr;
+            Points[0]=PositionX;	Points[1]=PositionY;
+            Points[2]=PositionX+dx;	Points[3]=PositionX+dy;
+            n = 2;
+            ClosePathLine(&Points,n,false,Tx);
+          }
+          else
+          {
+	    PositionX += dx;
+	    PositionY += dy;
+          }
+          //VectorEllipse *pVectCirc = new VectorEllipse(cy-r, cy+r, cx+r, cx-r);
+        }
+        if(err!=NULL)
+            fprintf(err,_("\nError: Path command 'a' is only partially supported!"));
+	break;
+
       case 'M':			//move to.
         if(n>0) SegmentCount++;
         if(isCurve)
@@ -1764,17 +2153,27 @@ VectorList VectListIn;
           ReadFloat(&StrMovements, Points[2*n-2]);
           if(*StrMovements==',') StrMovements++;
           while(isspace(*StrMovements)) StrMovements++;
-          if(isdigit(*StrMovements) || *StrMovements=='.' || *StrMovements=='-')
+          if(isdigit(*StrMovements) || *StrMovements=='.' || *StrMovements=='-' || !strnicmp(StrMovements,"nan",3))
           {
             ReadFloat(&StrMovements, Points[2*n-1]);
           }
           else
             Points[2*n-1] = PositionY - PositionOffsetY;
 
-          Points[2*n-2] += PositionOffsetX;
-          Points[2*n-1] += PositionOffsetY;
-          PositionX = Points[2*n-2];
-          PositionY = Points[2*n-1];
+	  if(IsFinite(&Points[2*n-2]))
+          {
+            Points[2*n-2] += PositionOffsetX;
+            PositionX = Points[2*n-2];
+          }
+          else
+            Points[2*n-2] = PositionX;
+          if(IsFinite(&Points[2*n-1]))
+          {
+            Points[2*n-1] += PositionOffsetY;          
+            PositionY = Points[2*n-1];
+          }
+          else
+            Points[2*n-1] = PositionY;
        break;
 
      case 'l':		//relative line to.
@@ -1800,14 +2199,16 @@ VectorList VectListIn;
           }
 
           ReadFloat(&StrMovements, val);
-          PositionX += val;
+          if(IsFinite(&val))
+              PositionX += val;
           Points[2*n-2] = PositionX;
           if(*StrMovements==',') StrMovements++;
           while(isspace(*StrMovements)) StrMovements++;
-          if(isdigit(*StrMovements) || *StrMovements=='.' || *StrMovements=='-')
+          if(isdigit(*StrMovements) || *StrMovements=='.' || *StrMovements=='-' || !strnicmp(StrMovements,"nan",3))
           {
             ReadFloat(&StrMovements, val);
-            PositionY += val;
+            if(IsFinite(&val))
+                PositionY += val;
           }
           Points[2*n-1] = PositionY;
        break;
@@ -2225,6 +2626,7 @@ VectorList VectListIn;
             if(GetProp("stroke-width",stroke_width))
             {
               pVectPath->PenWidth = stroke_width;
+              if(pVectPath->LineStyle==0) pVectPath->LineStyle=1;
             }
             switch(GetColor(pVectPath->FillColor,FILL_COLOR))
             {
@@ -2240,7 +2642,7 @@ VectorList VectListIn;
             GetColor(pVectPath->LineColor);
             GetLineCap(pVectPath->LineCap);
             GetLineJoin(pVectPath->LineJoin);
-            //pVectPath->Close = Close;
+            pVectPath->Close = true;	//Close;
 
             pVectPath->Swap(VectList);
             VectListIn.AddObject(pVectPath);
@@ -2278,6 +2680,7 @@ VectorList VectListIn;
       if(GetProp("stroke-width",stroke_width))
       {
         pVectPath->PenWidth = stroke_width;
+        if(pVectPath->LineStyle==0) pVectPath->LineStyle=1;
       }
       switch(GetColor(pVectPath->FillColor,FILL_COLOR))
       {
@@ -2302,8 +2705,10 @@ VectorList VectListIn;
   if(Tx) delete Tx;
 
 #ifdef _DEBUG
+/*
   FILE*F=fopen("O:\\temp\\39\\dump.txt","wb");
   if(F) {VectList.Dump2File(F);fclose(F);}
+*/
 #endif
 
   return;
@@ -2369,8 +2774,15 @@ int n;
     {
       pVecPoly->PenWidth = stroke_width;
     }
-    GetColor(pVecPoly->LineColor);
-    GetColor(pVecPoly->FillColor,FILL_COLOR);
+    GetColor(pVecPoly->LineColor);    
+    switch(GetColor(pVecPoly->FillColor,FILL_COLOR))
+    {
+      case 0: pVecPoly->BrushStyle=FILL_NONE;
+              break;
+      case 1: if(pVecPoly->BrushStyle==FILL_NONE) pVecPoly->BrushStyle=FILL_SOLID;
+              break;    
+      //default:
+    }
     GetLineCap(pVecPoly->LineCap);
     pVecPoly->Close = true;
     VectList.AddObject(pVecPoly);
@@ -2413,7 +2825,14 @@ AbstractTransformXY *Tx = NULL;
     pVectRec->PenWidth = stroke_width;
   }
   GetColor(pVectRec->LineColor);
-  GetColor(pVectRec->FillColor,FILL_COLOR);
+  switch(GetColor(pVectRec->FillColor,FILL_COLOR))
+  {
+    case 0: pVectRec->BrushStyle=FILL_NONE;
+            break;
+    case 1: if(pVectRec->BrushStyle==FILL_NONE) pVectRec->BrushStyle=FILL_SOLID;
+            break;    
+      //default:
+  }
 
   if(Tx && pVectRec->Tx==NULL)
   {
@@ -2438,7 +2857,6 @@ void TconvertedPass1_SVG::Defs(void)
 #ifdef DEBUG
   fprintf(log,"\n#TconvertedPass1_SVG::Defs ");fflush(log);
 #endif
-static const char ObjName[] = "!Defs";
 int i;
 FloatBBox bbxBk = bbx;	// Defines should not update current bounding box.
 
@@ -2448,7 +2866,40 @@ FloatBBox bbxBk = bbx;	// Defines should not update current bounding box.
   {
     ReadXMLTag(false);
 
-    if(by==XML_tag && TAG=="<linearGradient>")
+    if(by==XML_tag && TAG=="<clipPath>")
+    {
+      DefinedObject *pDefObj = NULL;
+      if((i=TAG_Args IN "id") >= 0)
+      {
+        const char * const Payload = TAG_Args.Member(i,1);
+        if(Payload!=NULL)
+        {
+          pDefObj = new DefinedObject(Payload);
+          if(pDefObj)
+          {
+            if(GetColor(pDefObj->FillColor,FILL_COLOR) == 1)
+                pDefObj->PropsSet |= DefinedObject::EpropFillColor;
+            if(GetColor(pDefObj->LineColor,LINE_COLOR) == 1)
+                pDefObj->PropsSet |= DefinedObject::EpropLineColor;
+          }
+        }
+      }
+      recursion++;
+      while(!feof(wpd))
+      {
+        if(pDefObj) VectList.Swap(pDefObj->VectList);
+        ProcessKeySVG();
+        if(pDefObj) VectList.Swap(pDefObj->VectList);
+        ReadXMLTag(false);
+
+        if(by==XML_closetag && (TAG=="</defs>" || TAG=="</clipPath>"))
+            break;
+      }
+      recursion--;
+      if(pDefObj) AddDefine(pDefObj);
+    }
+
+    if(by==XML_tag && (TAG=="<linearGradient>" || TAG=="<radialGradient>"))
     {
       DefinedObject *pDefObj = NULL;
       if((i=TAG_Args IN "id") >= 0)
@@ -2470,12 +2921,12 @@ FloatBBox bbxBk = bbx;	// Defines should not update current bounding box.
       recursion++;
       while(!feof(wpd))
       {
-        VectList.Swap(pDefObj->VectList);
+        if(pDefObj) VectList.Swap(pDefObj->VectList);
         ProcessKeySVG();
-        VectList.Swap(pDefObj->VectList);
+        if(pDefObj) VectList.Swap(pDefObj->VectList);
         ReadXMLTag(false);
 
-        if(by==XML_closetag && (TAG=="</defs>" || TAG=="</linearGradient>"))
+        if(by==XML_closetag && (TAG=="</defs>" || TAG=="</linearGradient>" || TAG=="</radialGradient>"))
             break;
         if(by==XML_tag && TAG=="<stop>")
         {          
@@ -2515,12 +2966,15 @@ FloatBBox bbxBk = bbx;	// Defines should not update current bounding box.
       }
     }
 
+    const unsigned char BkFlag=flag;
+    flag = Nothing;		// Prevent to expand anything inside Defs.
     ProcessKeySVG();
+    flag = BkFlag;
   }
   recursion--;
   bbx = bbxBk;
 
-  strcpy(ObjType,ObjName+1);
+  strcpy(ObjType,"Defs");
 }
 
 
@@ -2646,6 +3100,7 @@ uint8_t by, subby;
 	       if(loc_TAG=="polygon") {by=SVG_POLYGON;break;}
 	       if(loc_TAG=="path")    {by=SVG_PATH;break;}
 	       if(loc_TAG=="g")       {by=SVG_G;break;}
+	       if(loc_TAG=="a")       {by=SVG_A;break;}
 	       if(loc_TAG=="text")    {by=SVG_TEXT;break;}
 	       if(loc_TAG=="defs")    {by=SVG_DEFS;break;}
 	       if(loc_TAG=="use")     {by=SVG_USE;break;}
@@ -2663,6 +3118,7 @@ uint8_t by, subby;
 
   this->by = by;
   this->subby = subby;
+
   if(flag<Nothing)
     switch(by)
 	{
@@ -2688,6 +3144,7 @@ uint8_t by, subby;
 	case SVG_POLYLINE:PolyLine(); break;
 	case SVG_POLYGON:Polygon(); break;
 	case SVG_PATH:Path(); break;
+	case SVG_A:Anchor(); break;
 	case SVG_G:Group(); break;
 	case SVG_TEXT:Text(); break;
 	case SVG_DEFS:Defs(); break;
@@ -2697,8 +3154,9 @@ uint8_t by, subby;
 	}
 
 
- this->by = by;
- this->subby = subby;		// restore local by & subby.
+ this->by = by;				// restore local by & subby.
+ this->subby = subby;
+
  if (log != NULL)
     {   /**/
     if(by==128)
@@ -2835,7 +3293,9 @@ int RetVal = 0;
     ProcessKeySVG();
   }
   SolveDelayedObjs();
-
+#ifdef _DEBUG
+  //{FILE *F=fopen("O:\\temp\\40\\svg_file.txt","wb");VectList.Dump2File(F);fclose(F);}
+#endif
   ProcessImage();
 
   if(!Img.isEmpty())
@@ -2910,6 +3370,9 @@ size_t fsize;
       ConvSvg.wpd = NULL;
 
       ConvSvg.SolveDelayedObjs();
+#ifdef _DEBUG
+     //{FILE *F=fopen("O:\\temp\\24\\svg_file.txt","wb");ConvSvg.VectList.Dump2File(F);fclose(F);}
+#endif
       ConvSvg.ProcessImage();
     }
   }
