@@ -129,7 +129,7 @@ Finish:
 }
 
 
-/// Read one XML object from wpd stream.
+/// Read one XML object from wpd stream. See ::XML_STATUS.
 /// cq->by = 0 (normal char), 1 (extended char &xxx;), 2 (tag <>), 3 (end tag </>), 4 comment, 5 unfinished &xxx, 
 ///          6 expanded unicode, 7 CDATA block, 127 fail
 void TconvertedPass1_XML::ReadXMLTag(bool MakeUpper)
@@ -280,11 +280,17 @@ void TconvertedPass1_XML::CommentXML(void)
 #ifdef DEBUG
   fprintf(log,"\n#CommentXML() ");fflush(log);
 #endif
+  static const char ObjName[] = "!Comment";
   signed char Old_char_on_line;
   unsigned char OldFlag;
   attribute OldAttr;
   int i,maxi,commentchars;
 
+  if(strip==NULL)
+  {
+    strcpy(ObjType, ObjName);
+    return;
+  }
 
   OldFlag = flag;
   OldAttr = attr;
@@ -354,7 +360,7 @@ void TconvertedPass1_XML::CommentXML(void)
 
 
   recursion--;
-  strcpy(ObjType, "Comment");
+  strcpy(ObjType, ObjName+1);
   attr = OldAttr;
   flag = OldFlag;
   if(Old_char_on_line==CHAR_PRESENT || Old_char_on_line==JUNK_CHARS) char_on_line = JUNK_CHARS;
