@@ -81,6 +81,7 @@ Flags:'h';       Hard return
 int8_t   Accents = STYLE_NOTUSED,
 	AmsMath = -1,
         Amssymb = -1,		// Totaly Disabled
+        arabtex = STYLE_NOTUSED,
 	Arevmath = -1,		// Totaly Disabled
         Cyrillic = STYLE_NOTUSED,
 	cjHebrew = STYLE_NOTUSED,
@@ -187,6 +188,7 @@ const TStyleOpCl StyleOpCl2[] =
   {"\\begin{cjhebrew}", "\\end{cjhebrew}", NULL, NULL},
   {"\\begin{linenumbers}", "\\end{linenumbers}", NULL, NULL},
   {"\\mxedc ", "", NULL, NULL},
+  {"\\begin{RLtext}", "\\end{RLtext}", NULL, NULL}
 };
 
 
@@ -224,6 +226,7 @@ const TStyleOpCl StyleOpCl3[] =
   {"\\begin{cjhebrew}", "\\end{cjhebrew}", NULL, NULL},
   {"\\begin{linenumbers}", "\\end{linenumbers}", NULL, NULL},
   {"\\mxedc ", "", NULL, NULL},
+  {"\\begin{RLtext}", "\\end{RLtext}", NULL, NULL}
 };
 
 
@@ -1542,42 +1545,43 @@ typedef struct
 /** Package list with command line switches. */
 static const PkgItem PkgList[] = 
 	{{"accents",&Accents, N_("Use accents to support additional accents.")},
-        {"amssymb",&Amssymb,NULL},
-        {"Amssymb",&Amssymb, N_("Use AMSsymb with special characters.")},
+	{"amssymb",&Amssymb,NULL},
+	{"Amssymb",&Amssymb, N_("Use AMSsymb with special characters.")},
+	{"arabtex",&arabtex, N_("Use arabtex.sty to support arrabic glyphs.")},
 	{"arevmath",&Arevmath,NULL},
-        {"Arevmath",&Arevmath, N_("Use arevmath with special characters.")},
-        {"amsmath",&AmsMath,NULL},
-        {"AmsMath",&AmsMath, N_("Use AmsMath with special characters.")},
-        {"bbm",&Bbm,NULL},
-        {"Bbm",&Bbm, N_("Use Bbm with special characters.")},
-        {"cjhebrew",&cjHebrew}, {"cjHebrew",&cjHebrew},
-        {"cyrillic",&Cyrillic,NULL},
-        {"Cyrillic",&Cyrillic, N_("Use cyrillic.sty to support cyrillic script.")},
-        {"endnotes",&EndNotes,NULL},
-        {"Endnotes",&EndNotes},
-        {"fancyhdr",&FancyHdr}, {"Fancyhdr",&FancyHdr},   {"FancyHdr",&FancyHdr},
-        {"fontenc-T1",&fontenc, NULL},
-        {"FontEnc-T1",&fontenc, N_("Use package fontenc.sty [T1] with additional characters")},
-        {"LaTeXsym",&LaTeXsym,NULL},
-        {"latexsym",&LaTeXsym, N_("Use latexsym with special characters.")},
-        {"LineNo", &LineNo,NULL},
-        {"lineno", &LineNo},
-        {"longtable",&LongTable,NULL},
-        {"LongTable",&LongTable, N_("Use longtable for better tables.")},
-        {"makeidx",&MakeIdx,NULL},   
-        {"MakeIdx",&MakeIdx, N_("Use makeidx for placing index.")},
-        {"mxedruli",&mxedruli, N_("Use mxedruli for Georgia characters.")},
-        {"pifont",&pifont,NULL},
-        {"PiFont",&pifont, N_("Use additional package pifont.")},
-        {"rsfs",&Rsfs,NULL},
-        {"Rsfs",&Rsfs, N_("Use mathrsfs with special characters.")},
-        {"scalerel",&Scalerel, N_("Use additional package scalerel.")},
-        {"tipa",&TIPA,NULL},
-        {"TIPA",&TIPA, N_("Use TIPA.sty for IPA and other phonetic characters")},
-        {"ulem",&Ulem,NULL},
-        {"Ulem",&Ulem, N_("Use ulem for underlining and striking out.")},
-        {"wasy",&Wasy,NULL},
-        {"Wasy",&Wasy,N_("Use Wasy2 with special characters.")}};
+	{"Arevmath",&Arevmath, N_("Use arevmath with special characters.")},
+	{"amsmath",&AmsMath,NULL},
+	{"AmsMath",&AmsMath, N_("Use AmsMath with special characters.")},
+	{"bbm",&Bbm,NULL},
+	{"Bbm",&Bbm, N_("Use Bbm with special characters.")},
+	{"cjhebrew",&cjHebrew}, {"cjHebrew",&cjHebrew},
+	{"cyrillic",&Cyrillic,NULL},
+	{"Cyrillic",&Cyrillic, N_("Use cyrillic.sty to support cyrillic script.")},
+	{"endnotes",&EndNotes,NULL},
+	{"Endnotes",&EndNotes},
+	{"fancyhdr",&FancyHdr}, {"Fancyhdr",&FancyHdr},   {"FancyHdr",&FancyHdr},
+	{"fontenc-T1",&fontenc, NULL},
+	{"FontEnc-T1",&fontenc, N_("Use package fontenc.sty [T1] with additional characters")},
+	{"LaTeXsym",&LaTeXsym,NULL},
+	{"latexsym",&LaTeXsym, N_("Use latexsym with special characters.")},
+	{"LineNo", &LineNo,NULL},
+	{"lineno", &LineNo},
+	{"longtable",&LongTable,NULL},
+	{"LongTable",&LongTable, N_("Use longtable for better tables.")},
+	{"makeidx",&MakeIdx,NULL},   
+	{"MakeIdx",&MakeIdx, N_("Use makeidx for placing index.")},
+	{"mxedruli",&mxedruli, N_("Use mxedruli for Georgia characters.")},
+	{"pifont",&pifont,NULL},
+	{"PiFont",&pifont, N_("Use additional package pifont.")},
+	{"rsfs",&Rsfs,NULL},
+	{"Rsfs",&Rsfs, N_("Use mathrsfs with special characters.")},
+	{"scalerel",&Scalerel, N_("Use additional package scalerel.")},
+	{"tipa",&TIPA,NULL},
+	{"TIPA",&TIPA, N_("Use TIPA.sty for IPA and other phonetic characters")},
+	{"ulem",&Ulem,NULL},
+	{"Ulem",&Ulem, N_("Use ulem for underlining and striking out.")},
+	{"wasy",&Wasy,NULL},
+	{"Wasy",&Wasy,N_("Use Wasy2 with special characters.")}};
 
 
 static char StyleUsed(int8_t *u)
@@ -1601,7 +1605,7 @@ void PrintPackageList(void)
 
 
 /// See which CPs are supported by inputenc: https://ctan.math.illinois.edu/macros/latex/base/inputenc.pdf
-const TOutCodePage OutCodepages[11] =
+const TOutCodePage OutCodepages[12] =
 {
   {"852", "internalTOcp852", 852, true},
   {"866", NULL, 866, false},
@@ -1613,7 +1617,8 @@ const TOutCodePage OutCodepages[11] =
   {"ISO8859-2", "internalTOiso_8859_2", ISO8859_2, true},
   {"ISO-8859-2", "internalTOiso_8859_2", ISO8859_2, true},
   {"KOI8R", NULL, KOI8R, false},
-  {"UTF8", "internalTOunicode", UTF8, true}
+  {"UTF8", "internalTOunicode", UTF8, true},
+  {"UTF-8", "internalTOunicode", UTF8, true}
 };
 
 
@@ -1681,11 +1686,11 @@ int p;
 		if(OutCodepages[i].Translator)
 		  Convert2Target = GetTranslator(OutCodepages[i].Translator);
 		CPStyles = OutCodepages[i].UseCpStyles;
-                break;	      
+                break;
                 }
 	      }
-          if(i<=sizeof(OutCodepages)/sizeof(OutCodePage)) continue;	// Output codepage has been found.
-	  
+          if(i<sizeof(OutCodepages)/sizeof(TOutCodePage)) continue;	// Output codepage has been found.
+
 	  if(!strcmp(name, "Styles") || !strcmp(name, "styles"))
 	     {
 	     CPStyles = true;
